#

bl_info = {
	"name": "Extrude to New Object",
	"author": "",
	"version": (0, 0, 1),
	"blender": (2, 72, 0),
	"location": "",
	"category": "Mesh",
};

import bpy;

VGROUP_NAME = "__extrude_to_new_object";
g_object = None;
g_vgroups = [];
g_marked_polys = [];

def flip_editmode():
	bpy.ops.object.mode_set(mode="OBJECT");
	bpy.ops.object.mode_set(mode="EDIT");

class ExtrudeToNewObjectStartOp(bpy.types.Operator):
	bl_idname = "mesh.extrude_to_new_object_start";
	bl_label = "Extrude to New Object Start";
	bl_options = {"UNDO"};

	@classmethod
	def poll(cls, context):
		return context.active_object and context.active_object.type == "MESH" \
				and context.mode == "EDIT_MESH";

	def execute(self, context):
		global g_vgroups;
		global g_object;
		global g_marked_polys;

		bpy.ops.object.mode_set(mode="OBJECT");
		sel_faces = [f.index for f in context.active_object.data.polygons if f.select];
		bpy.ops.object.mode_set(mode="EDIT");
		if(len(sel_faces) < 1):
			return {"CANCELLED"};

		g_vgroups = [];
		g_marked_polys = [];
		o = context.active_object;
		g_object = o;

		bpy.ops.object.mode_set(mode="OBJECT");

		"""
		# create vgroups & assign verts
		for i, pi in enumerate(sel_faces):
			p = o.data.polygons[pi];
			o.vertex_groups.new(VGROUP_NAME);
			vgroup = o.vertex_groups[-1];
			pv = [o.data.vertices[v].index for v in p.vertices];
			vgroup.add(pv, 1.0, "ADD");
			g_marked_polys.append(pi);

			g_vgroups.append(vgroup);
		"""
		ekeys = {};
		for e in o.data.edges:
			ekeys[e.key] = e.index;

		for i, pi in enumerate(sel_faces):
			base_vertex = None;
			for l in range(len(o.data.vertices)):
				if(l not in o.data.polygons[pi].vertices):
					base_vertex = o.data.vertices[l].co;
					break;

			eidxs = [];
			vco = [];
			for v in o.data.polygons[pi].vertices:
				vt = o.data.vertices[v];
				vco.append(vt.co[:]);
			for ek in o.data.polygons[pi].edge_keys:
				e = o.data.edges[ekeys[ek]];
				if e.select:
					eidxs.append(e.index);
			g_marked_polys.append([pi, o.data.polygons[pi].vertices[:], vco, eidxs, base_vertex]);

		bpy.ops.object.mode_set(mode="EDIT");

		return {"FINISHED"};

class ExtrudeToNewObjectFinishOp(bpy.types.Operator):
	bl_idname = "mesh.extrude_to_new_object_finish";
	bl_label = "Extrude to New Object Finish";
	bl_description = "";
	bl_options = {"UNDO"};

	@classmethod
	def poll(cls, context):
		return context.active_object and context.active_object.type == "MESH" \
				and context.mode == "EDIT_MESH"\
				#and g_object == context.active_object;

	def execute(self, context):
		global g_object;
		global g_vgroups;
		global g_marked_polys;

		o = context.active_object;
		ao = o;
		created_objects = [];

		for mp in g_marked_polys:
			bpy.ops.object.mode_set(mode="EDIT");
			bpy.ops.mesh.select_all(action="DESELECT");
			bpy.ops.mesh.select_mode(type="VERT");
			bpy.ops.object.mode_set(mode="OBJECT");

			#for vidx in mp[1]:
			#	o.data.vertices[vidx].select = True;
			nselected = 0;
			for v in o.data.vertices:
				for vco in mp[2]:
					if vco[0] == v.co.x and vco[1] == v.co.y and vco[2] == v.co.z:
						v.select = True;
						nselected += 1;

			print("nselected=%d (len=%d/%d)" % (nselected, len(mp[2]), len(o.data.vertices)));

			bpy.ops.object.mode_set(mode="EDIT");
			bpy.ops.mesh.rip("INVOKE_DEFAULT");

			bpy.ops.mesh.select_linked("INVOKE_DEFAULT");

			internal = False;
			flip_editmode();
			for v in o.data.vertices:
				if v.select:
					if v.co == mp[4]:
						print("internal");
						internal = True;
					break;

			if internal:
				bpy.ops.mesh.select_all(action="INVERT");

			#bpy.ops.mesh.loop_to_region();
			bpy.ops.mesh.separate(type="SELECTED")
			bpy.ops.mesh.select_all(action="DESELECT");

			# find separated obj
			bpy.ops.object.mode_set(mode="OBJECT");
			for obj in context.selected_objects:
				if obj != o and obj not in created_objects:
					created_objects.append(obj);
					break;

			bpy.ops.object.mode_set(mode="OBJECT");
			for v in o.data.vertices:
				for vco in mp[2]:
					if vco[0] == v.co.x and vco[1] == v.co.y and vco[2] == v.co.z:
						v.select = True;
			#for vidx in mp[1]:
			#	o.data.vertices[vidx].select = True;
			flip_editmode();
			bpy.ops.mesh.edge_face_add("INVOKE_DEFAULT");
			#flip_editmode();

		bpy.ops.object.mode_set(mode="OBJECT");
		for i, o in enumerate(created_objects):
			bpy.context.scene.objects.active = o;
			mp = g_marked_polys[i];
			for v in o.data.vertices:
				for vco in mp[2]:
					if vco[0] == v.co.x and vco[1] == v.co.y and vco[2] == v.co.z:
						v.select = True;
			bpy.ops.object.mode_set(mode="EDIT");
			bpy.ops.mesh.edge_face_add("INVOKE_DEFAULT");
			bpy.ops.object.mode_set(mode="OBJECT");

		g_object = None;
		g_vgroups = [];
		g_marked_polys = [];

		return {"FINISHED"};

class ExtrudeToNewObjectPanel(bpy.types.Panel):
	bl_label = "Extrude to New Object";
	#bl_idname = "ExtrudeToNewObject";
	bl_space_type = "VIEW_3D";
	bl_region_type = "UI";
	#bl_category = "Tools";

	@classmethod
	def poll(self, context):
		return context.active_object and context.active_object.type == "MESH" \
				and context.mode == "EDIT_MESH";

	def draw(self, context):
		self.layout.operator(ExtrudeToNewObjectStartOp.bl_idname);
		self.layout.operator(ExtrudeToNewObjectFinishOp.bl_idname);

register_classes = (
	ExtrudeToNewObjectStartOp,
	ExtrudeToNewObjectFinishOp,
	ExtrudeToNewObjectPanel,
);

addon_keymaps = [];

def register():
	for c in register_classes:
		bpy.utils.register_class(c);

	# keys
	global addon_keymaps;
	wm = bpy.context.window_manager;
	km = wm.keyconfigs.addon.keymaps.new(name="3D View", space_type="VIEW_3D");
	kmi = km.keymap_items.new(ExtrudeToNewObjectStartOp.bl_idname, "Q", "RELEASE", alt=True);
	addon_keymaps.append((km, kmi));
	kmi = km.keymap_items.new(ExtrudeToNewObjectFinishOp.bl_idname, "W", "RELEASE", alt=True);
	addon_keymaps.append((km, kmi));

def unregister():
	global addon_keymaps;
	for km, kmi in addon_keymaps:
		km.keymap_items.remove(kmi);
	addon_keymaps.clear();

	for c in register_classes:
		bpy.utils.unregister_class(c);
